#include <iostream>
#include <chrono>
#include "rocketmq/ONSFactory.h"

using namespace std;
using namespace ons;

int main() {
    std::cout << "=======Before sending messages=======" << std::endl;
    ONSFactoryProperty factoryInfo;
    //Request from ONS console, this should be GroupID here.
    //请填写在阿里云ONS控制台上申请的GroupID，从实例化的版本开始，ProducerId和CounsumerId已经统一，此处设置是为了接口保持向前兼容。
    factoryInfo.setFactoryProperty(ONSFactoryProperty::ProducerId, "GID_XXX");
    //Request from ONS console
    //Your Access Key from your account.
    //请填写你的账户的AK
    factoryInfo.setFactoryProperty(ONSFactoryProperty::AccessKey, "Your Access Key");
    //Your Secret Key from your account.
    //请填写你的账户的SK
    factoryInfo.setFactoryProperty(ONSFactoryProperty::SecretKey, "Your Secret Key");
    //This is the endpoint from ONS console
    //请填写阿里云ONS控制台上对应实例的接入点
    factoryInfo.setFactoryProperty(ONSFactoryProperty::NAMESRV_ADDR,
                                   "http://xxxxxxxxxxxxxxxx.aliyuncs.com:80");
    const char* topic = "Your-topic";
    Producer *producer = nullptr;
    producer = ONSFactory::getInstance()->createProducer(factoryInfo);
    producer->start();
    Message msg(
            topic,
            "Your Tag",
            "Your Key",
            "This message body."
    );

    auto start = std::chrono::system_clock::now();
    int count = 32;
    for (int i = 0; i < count; ++i) {
        try {
            SendResultONS sendResult = producer->send(msg);
            std::cout << "Message ID: " << sendResult.getMessageId() << std::endl;
        } catch (const ONSClientException& e) {
            std::cout << "Failed to send message. Cause: " << e.what() << std::endl;
        }

        // If prefer not to use exception
        {
            std::error_code ec;
            SendResultONS send_result = producer->send(msg, ec);
            if (!ec) {
                std::cout << "Send message[msgId=" << send_result.getMessageId() << "] OK\n";
            } else {
                std::cerr << "Failed to send message[msgId=" << msg.getMsgID() << "]. Caused by: " << ec.message() <<
                 " ErrorCode=" << ec.value() << std::endl;
            }
        }
    }
    auto interval = std::chrono::system_clock::now() - start;
    std::cout << "Sending completed, costing "
              << std::chrono::duration_cast<std::chrono::milliseconds>(interval).count() << "ms" << std::endl;

    //Keep main thread running until process finished.
    //请保持线程常驻，不要执行shutdown操作
    producer->shutdown();
    std::cout << "=======After sending messages=======" << std::endl;
    return 0;
}