#pragma once

#include <exception>
#include <string>
#include <system_error>

#include "ONSClient.h"

ROCKETMQ_NAMESPACE_BEGIN

enum ONSError : int32_t {
    SUCCESS = 0,
    NETWORK_UNREACHABLE = 1,
    IO_TIMEOUT = 2,
    BAD_MESSAGE = 3,
    TOPIC_NOT_AVAILABLE = 4,
    TOPIC_REQUIRED = 5,
    SERVER_INTERVAL_ERROR = 6,
    CALL_BACK_REQUIRED = 7,
    MESSAGE_QUEUE_SELECTOR_REQUIRED = 8,
    LOCAL_EXECUTOR_REQUIRED = 9,
    BODY_SIZE_TOO_LARGE = 10,
    GENERIC_ERROR = 100,
    BAD_INSTANCE_INDEX = 500
};

class ONSCategory : public std::error_category {
   public:
    const char* name() const noexcept override;

    std::string message(int error_value) const noexcept override;

    std::error_condition default_error_condition(int ev) const noexcept override;

};

const std::error_category& ons_category();

std::error_code make_error_code(ONSError e);

std::error_condition make_error_condition(ONSError e);

class ONSCLIENT_API ONSClientException : public std::exception {
   public:
    ONSClientException();

    ONSClientException(std::string msg, int error) noexcept;

    ~ONSClientException() override = default;

    ONSClientException(const ONSClientException& other) noexcept;

    const char* GetMsg() const noexcept;

    const char* what() const noexcept override;

    int GetError() const noexcept;

   private:
    std::string m_msg;
    int m_error;
};

ROCKETMQ_NAMESPACE_END

namespace std {
template <>
struct is_error_code_enum<ons::ONSError> : public true_type {};
}  // namespace std