#pragma once

#include "LocalTransactionChecker.h"
#include "ONSChannel.h"
#include "ONSClientException.h"
#include "OrderConsumer.h"
#include "OrderProducer.h"
#include "Producer.h"
#include "PullConsumer.h"
#include "PushConsumer.h"
#include "TransactionProducer.h"

ROCKETMQ_NAMESPACE_BEGIN

class ONSCLIENT_API ONSFactoryProperty {
 public:
  ONSFactoryProperty();

  ~ONSFactoryProperty() = default;

  static bool checkValidityOfFactoryProperties(const std::string& key, const std::string& value) noexcept(false);
  const char* getLogPath() const;
  void setSendMsgTimeout(int value);
  void setSendMsgRetryTimes(int value);
  void setMaxMsgCacheSize(int size);
  void setOnsTraceSwitch(bool onsSwitch);
  void setOnsChannel(ONSChannel onsChannel) noexcept(false);
  void setFactoryProperty(const char* key, const char* value) noexcept(false);
  void remove(const char* key) noexcept;
  void setFactoryProperties(const std::map<std::string, std::string>& factoryProperties);
  std::map<std::string, std::string> getFactoryProperties() const;
  const char* getProducerId() const;
  const char* getConsumerId() const;
  const char* getGroupId() const;
  const char* getPublishTopics() const;
  const char* getMessageModel() const;
  int getSendMsgTimeout() const;
  int getSuspendTimeMillis() const;
  int getSendMsgRetryTimes() const;
  int getConsumeThreadNums() const;
  int getMaxMsgCacheSize() const;
  int getMaxMsgCacheSizeInMiB() const;
  ONSChannel getOnsChannel() const;
  const char* getChannel() const;
  const char* getMessageContent() const;
  const char* getNameSrvAddr() const;
  const char* getNameSrvDomain() const;
  const char* getAccessKey() const;
  const char* getSecretKey() const;
  const char* getConsumerInstanceName() const;
  bool getOnsTraceSwitch() const;
  const char* getInstanceId() const;
  LanguageIdentifier languageIdentifier() const;
  void languageIdentifier(LanguageIdentifier identifier);

 public:
  static const char* LogPath;
  static const char* ProducerId;
  static const char* ConsumerId;
  static const char* GroupId;
  static const char* PublishTopics;
  static const char* MsgContent;
  static const char* ONSAddr;
  static const char* AccessKey;
  static const char* SecretKey;
  static const char* MessageModel;
  static const char* BROADCASTING;
  static const char* CLUSTERING;
  static const char* SendMsgTimeoutMillis;
  static const char* SuspendTimeMillis;
  static const char* NAMESRV_ADDR;
  static const char* ConsumeThreadNums;
  static const char* OnsChannel;
  static const char* MaxMsgCacheSize;
  static const char* MaxCachedMessageSizeInMiB;
  static const char* OnsTraceSwitch;
  static const char* SendMsgRetryTimes;
  static const char* ConsumerInstanceName;
  static const char* InstanceId;
  static const char* LanguageIdentifierName;

  static const char* HOME_ENV;
  static const char* DEFAULT_CONFIG_FILE_NAME;

 private:
  std::map<std::string, std::string> m_onsFactoryProperties;
};

class ONSCLIENT_API ONSFactoryAPI {
 public:
  ONSFactoryAPI() = default;
  virtual ~ONSFactoryAPI() = default;

  virtual ons::Producer* createProducer(ons::ONSFactoryProperty factory_property) noexcept(false) = 0;
  virtual ons::OrderProducer* createOrderProducer(ons::ONSFactoryProperty factory_property) noexcept(false) = 0;
  virtual ons::OrderConsumer* createOrderConsumer(ons::ONSFactoryProperty factory_property) noexcept(false) = 0;
  virtual ons::TransactionProducer* createTransactionProducer(ons::ONSFactoryProperty factory_property,
                                                              ons::LocalTransactionChecker* checker) noexcept(false) = 0;
  virtual ons::PullConsumer* createPullConsumer(ons::ONSFactoryProperty factory_property) noexcept(false) = 0;
  virtual ons::PushConsumer* createPushConsumer(ons::ONSFactoryProperty factory_property) noexcept(false) = 0;
};

class ONSCLIENT_API ONSFactory {
 public:
  ONSFactory() = delete;

  ~ONSFactory() = default;

  static ons::ONSFactoryAPI* getInstance();
};

ROCKETMQ_NAMESPACE_END