#pragma once

#include "Message.h"
#include "MessageQueueONS.h"
#include "MessageQueueSelectorONS.h"
#include "ONSCallback.h"
#include "ONSClientException.h"
#include "SendResultONS.h"

ROCKETMQ_NAMESPACE_BEGIN

class ONSCLIENT_API Producer {
   public:
    Producer() = default;

    virtual ~Producer() = default;

    // before send msg, start must be called to allocate resources.
    virtual void start() = 0;

    // before exit ons, shutdown must be called to release all resources allocated
    // by ons internally.
    virtual void shutdown() = 0;

    // retry at most 3 times before claim sending failure. if no exception were thrown, it is safe to assume sending is
    // successful;
    virtual SendResultONS send(Message& msg) noexcept(false) = 0;

    virtual SendResultONS send(Message& msg, std::error_code& tc) = 0;

    virtual SendResultONS send(Message& msg, const MessageQueueONS& mq) noexcept(false) = 0;

    virtual SendResultONS send(Message& msg, const MessageQueueONS& mq, std::error_code& ec) = 0;

    // async send
    virtual void sendAsync(Message& msg, SendCallbackONS* callback) noexcept(false) = 0;

    virtual void sendAsync(Message& msg, SendCallbackONS* callback, std::error_code& ec) = 0;

    // oneway send
    virtual void sendOneway(Message& msg) noexcept(false) = 0;

    virtual void sendOneway(Message& msg, std::error_code& ec) = 0;

    virtual SendResultONS send(Message& msg, MessageQueueSelectorONS* selector, void* arg) noexcept(false) = 0;

    virtual SendResultONS send(Message& msg, MessageQueueSelectorONS* selector, void* arg,
                               std::error_code& ec) = 0;
};

ROCKETMQ_NAMESPACE_END